@@ -1,5 +1,5 @@
 package Scientist;
-$Scientist::VERSION = '0.005';
+$Scientist::VERSION = '0.006';
 use strict;
 use warnings;
 
@@ -23,31 +23,54 @@ has 'result' => ( is => 'rw', );
 
 has 'try' => ( is => 'rw' );
 
+sub publish {
+    my $self = shift;
+    # Stub publish sub, extend this to enable your own own
+    # unique publishing requirements
+    return;
+}
+
 sub run {
     my $self = shift;
-    my %result;
 
-    if ( !$self->enabled ) {
-        # If experiement not enabled just return the control
-        # code results
-        return $self->use->();
+    # If experiement not enabled just return the control code results.
+    return $self->use->() unless $self->enabled;
+
+    my %result = (
+        context    => $self->context,
+        experiment => $self->experiment,
+    );
+
+    my $wantarray = wantarray;
+
+    my ( @control, @candidate );
+    my $run_control = sub {
+        my $start = Time::HiRes::time;
+        @control = $wantarray ? $self->use->() : scalar $self->use->();
+        $result{control}{duration} = Time::HiRes::time - $start;
+    };
+
+    my $run_candidate = sub {
+        my $start = Time::HiRes::time;
+        @candidate = $wantarray ? $self->try->() : scalar $self->try->();
+        $result{candidate}{duration} = Time::HiRes::time - $start;
+    };
+
+    if ( rand > 0.5 ) {
+        $run_control->();
+        $run_candidate->();
+    }
+    else {
+        $run_candidate->();
+        $run_control->();
     }
 
-    $result{context}    = $self->context;
-    $result{experiment} = $self->experiment;
-
-    my $start   = Time::HiRes::time;
-    my $control = $self->use->();
-    $result{control}{duration} = ( Time::HiRes::time - $start );
-
-    $start = Time::HiRes::time;
-    my $candidate = eval { $self->try->() };
-    $result{mismatched} = !eq_deeply( \$control, \$candidate ) ? 1 : 0;
-    $result{candidate}{duration} = ( Time::HiRes::time - $start );
+    $result{mismatched} = !eq_deeply( \@candidate, \@control ) ? 1 : 0;
 
     $self->result( \%result );
+    $self->publish;
 
-    return $control;
+    return $wantarray ? @control : $control[0];
 }
 
 1;
@@ -55,13 +78,13 @@ sub run {
 =head1 LICENSE
 
 This software is Copyright (c) 2016 by Lance Wicks.
- 
+
 This is free software, licensed under:
- 
+
   The MIT (X11) License
- 
+
 The MIT License
- 
+
 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated
 documentation files (the "Software"), to deal in the Software
@@ -70,11 +93,11 @@ use, copy, modify, merge, publish, distribute, sublicense,
 and/or sell copies of the Software, and to permit persons to
 whom the Software is furnished to do so, subject to the
 following conditions:
- 
+
 The above copyright notice and this permission notice shall
 be included in all copies or substantial portions of the
 Software.
- 
+
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT
 WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
